@@ -26,8 +26,6 @@
 
 #include <rfb/Logger.h>
 
-namespace os { class Mutex; }
-
 namespace rfb {
 
   class Logger_File : public Logger {
@@ -35,7 +33,7 @@ namespace rfb {
     Logger_File(const char* loggerName);
     ~Logger_File();
 
-    virtual void write(int level, const char *logname, const char *message);
+    void write(int level, const char *logname, const char *message) override;
     void setFilename(const char* filename);
     void setFile(FILE* file);
 
@@ -47,7 +45,6 @@ namespace rfb {
     char m_filename[PATH_MAX];
     FILE* m_file;
     time_t m_lastLogTime;
-    os::Mutex* mutex;
   };
 
   bool initFileLogger(const char* filename);
