@@ -16,8 +16,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  * USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 
+#include <vector>
+
 #include <rdr/InStream.h>
 #include <rdr/ZlibInStream.h>
 
@@ -25,11 +32,11 @@
 #include <rfb/qemuTypes.h>
 #include <rfb/clipboardTypes.h>
 #include <rfb/Exception.h>
-#include <rfb/util.h>
 #include <rfb/SMsgHandler.h>
 #include <rfb/SMsgReader.h>
 #include <rfb/Configuration.h>
 #include <rfb/LogWriter.h>
+#include <rfb/util.h>
 
 using namespace rfb;
 
@@ -38,7 +45,7 @@ static LogWriter vlog("SMsgReader");
 static IntParameter maxCutText("MaxCutText", "Maximum permitted length of an incoming clipboard update", 256*1024);
 
 SMsgReader::SMsgReader(SMsgHandler* handler_, rdr::InStream* is_)
-  : handler(handler_), is(is_)
+  : handler(handler_), is(is_), state(MSGSTATE_IDLE)
 {
 }
 
@@ -46,78 +53,117 @@ SMsgReader::~SMsgReader()
 {
 }
 
-void SMsgReader::readClientInit()
+bool SMsgReader::readClientInit()
 {
+  if (!is->hasData(1))
+    return false;
   bool shared = is->readU8();
   handler->clientInit(shared);
+  return true;
 }
 
-void SMsgReader::readMsg()
+bool SMsgReader::readMsg()
 {
-  int msgType = is->readU8();
-  switch (msgType) {
+  bool ret;
+
+  if (state == MSGSTATE_IDLE) {
+    if (!is->hasData(1))
+      return false;
+
+    currentMsgType = is->readU8();
+    state = MSGSTATE_MESSAGE;
+  }
+
+  switch (currentMsgType) {
   case msgTypeSetPixelFormat:
-    readSetPixelFormat();
+    ret = readSetPixelFormat();
     break;
   case msgTypeSetEncodings:
-    readSetEncodings();
+    ret = readSetEncodings();
     break;
   case msgTypeSetDesktopSize:
-    readSetDesktopSize();
+    ret = readSetDesktopSize();
     break;
   case msgTypeFramebufferUpdateRequest:
-    readFramebufferUpdateRequest();
+    ret = readFramebufferUpdateRequest();
     break;
   case msgTypeEnableContinuousUpdates:
-    readEnableContinuousUpdates();
+    ret = readEnableContinuousUpdates();
     break;
   case msgTypeClientFence:
-    readFence();
+    ret = readFence();
     break;
   case msgTypeKeyEvent:
-    readKeyEvent();
+    ret = readKeyEvent();
     break;
   case msgTypePointerEvent:
-    readPointerEvent();
+    ret = readPointerEvent();
     break;
   case msgTypeClientCutText:
-    readClientCutText();
+    ret = readClientCutText();
     break;
   case msgTypeQEMUClientMessage:
-    readQEMUMessage();
+    ret = readQEMUMessage();
     break;
   default:
-    vlog.error("unknown message type %d", msgType);
-    throw Exception("unknown message type");
+    vlog.error("Unknown message type %d", currentMsgType);
+    throw protocol_error("Unknown message type");
   }
+
+  if (ret)
+    state = MSGSTATE_IDLE;
+
+  return ret;
 }
 
-void SMsgReader::readSetPixelFormat()
+bool SMsgReader::readSetPixelFormat()
 {
+  if (!is->hasData(3 + 16))
+    return false;
   is->skip(3);
   PixelFormat pf;
   pf.read(is);
   handler->setPixelFormat(pf);
+  return true;
 }
 
-void SMsgReader::readSetEncodings()
+bool SMsgReader::readSetEncodings()
 {
+  if (!is->hasData(1 + 2))
+    return false;
+
+  is->setRestorePoint();
+
   is->skip(1);
+
   int nEncodings = is->readU16();
-  rdr::S32Array encodings(nEncodings);
-  for (int i = 0; i < nEncodings; i++)
-    encodings.buf[i] = is->readU32();
-  handler->setEncodings(nEncodings, encodings.buf);
+
+  if (!is->hasDataOrRestore(nEncodings * 4))
+    return false;
+  is->clearRestorePoint();
+
+  std::vector<int32_t> encodings(nEncodings);
+  for (size_t i = 0; i < encodings.size(); i++)
+    encodings[i] = is->readU32();
+
+  handler->setEncodings(nEncodings, encodings.data());
+
+  return true;
 }
 
-void SMsgReader::readSetDesktopSize()
+bool SMsgReader::readSetDesktopSize()
 {
   int width, height;
   int screens, i;
-  rdr::U32 id, flags;
+  uint32_t id, flags;
   int sx, sy, sw, sh;
   ScreenSet layout;
 
+  if (!is->hasData(1 + 2 + 2 + 1 + 1))
+    return true;
+
+  is->setRestorePoint();
+
   is->skip(1);
 
   width = is->readU16();
@@ -126,6 +172,10 @@ void SMsgReader::readSetDesktopSize()
   screens = is->readU8();
   is->skip(1);
 
+  if (!is->hasDataOrRestore(screens * (4 + 2 + 2 + 2 + 2 + 4)))
+    return false;
+  is->clearRestorePoint();
+
   for (i = 0;i < screens;i++) {
     id = is->readU32();
     sx = is->readU16();
@@ -138,23 +188,31 @@ void SMsgReader::readSetDesktopSize()
   }
 
   handler->setDesktopSize(width, height, layout);
+
+  return true;
 }
 
-void SMsgReader::readFramebufferUpdateRequest()
+bool SMsgReader::readFramebufferUpdateRequest()
 {
+  if (!is->hasData(1 + 2 + 2 + 2 + 2))
+    return false;
   bool inc = is->readU8();
   int x = is->readU16();
   int y = is->readU16();
   int w = is->readU16();
   int h = is->readU16();
   handler->framebufferUpdateRequest(Rect(x, y, x+w, y+h), inc);
+  return true;
 }
 
-void SMsgReader::readEnableContinuousUpdates()
+bool SMsgReader::readEnableContinuousUpdates()
 {
   bool enable;
   int x, y, w, h;
 
+  if (!is->hasData(1 + 2 + 2 + 2 + 2))
+    return false;
+
   enable = is->readU8();
 
   x = is->readU16();
@@ -163,81 +221,145 @@ void SMsgReader::readEnableContinuousUpdates()
   h = is->readU16();
 
   handler->enableContinuousUpdates(enable, x, y, w, h);
+
+  return true;
 }
 
-void SMsgReader::readFence()
+bool SMsgReader::readFence()
 {
-  rdr::U32 flags;
-  rdr::U8 len;
-  char data[64];
+  uint32_t flags;
+  uint8_t len;
+  uint8_t data[64];
+
+  if (!is->hasData(3 + 4 + 1))
+    return false;
+
+  is->setRestorePoint();
 
   is->skip(3);
 
   flags = is->readU32();
 
   len = is->readU8();
+
+  if (!is->hasDataOrRestore(len))
+    return false;
+  is->clearRestorePoint();
+
   if (len > sizeof(data)) {
     vlog.error("Ignoring fence with too large payload");
     is->skip(len);
-    return;
+    return true;
   }
 
   is->readBytes(data, len);
   
   handler->fence(flags, len, data);
+
+  return true;
 }
 
-void SMsgReader::readKeyEvent()
+bool SMsgReader::readKeyEvent()
 {
+  if (!is->hasData(1 + 2 + 4))
+    return false;
   bool down = is->readU8();
   is->skip(2);
-  rdr::U32 key = is->readU32();
+  uint32_t key = is->readU32();
   handler->keyEvent(key, 0, down);
+  return true;
 }
 
-void SMsgReader::readPointerEvent()
+bool SMsgReader::readPointerEvent()
 {
-  int mask = is->readU8();
-  int x = is->readU16();
-  int y = is->readU16();
+  int mask;
+  int x;
+  int y;
+
+  if (!is->hasData(1 + 2 + 2))
+    return false;
+
+  is->setRestorePoint();
+
+  mask = is->readU8();
+  x = is->readU16();
+  y = is->readU16();
+
+  if (handler->client.supportsExtendedMouseButtons() && mask & 0x80 ) {
+    int highBits;
+    int lowBits;
+
+    if (!is->hasDataOrRestore(1))
+      return false;
+
+    highBits = is->readU8();
+    lowBits = mask & 0x7f; /* Clear marker bit */
+    mask = (highBits << 7) | lowBits;
+  }
+
+  is->clearRestorePoint();
   handler->pointerEvent(Point(x, y), mask);
+  return true;
 }
 
 
-void SMsgReader::readClientCutText()
+bool SMsgReader::readClientCutText()
 {
+  if (!is->hasData(3 + 4))
+    return false;
+
+  is->setRestorePoint();
+
   is->skip(3);
-  rdr::U32 len = is->readU32();
+  uint32_t len = is->readU32();
 
   if (len & 0x80000000) {
-    rdr::S32 slen = len;
+    int32_t slen = len;
     slen = -slen;
-    readExtendedClipboard(slen);
-    return;
+    if (readExtendedClipboard(slen)) {
+      is->clearRestorePoint();
+      return true;
+    } else {
+      is->gotoRestorePoint();
+      return false;
+    }
   }
 
+  if (!is->hasDataOrRestore(len))
+    return false;
+  is->clearRestorePoint();
+
   if (len > (size_t)maxCutText) {
     is->skip(len);
     vlog.error("Cut text too long (%d bytes) - ignoring", len);
-    return;
+    return true;
   }
-  CharArray ca(len);
-  is->readBytes(ca.buf, len);
-  CharArray filtered(convertLF(ca.buf, len));
-  handler->clientCutText(filtered.buf);
+
+  std::vector<char> ca(len);
+  is->readBytes((uint8_t*)ca.data(), len);
+
+  std::string utf8(latin1ToUTF8(ca.data(), ca.size()));
+  std::string filtered(convertLF(utf8.data(), utf8.size()));
+
+  handler->clientCutText(filtered.c_str());
+
+  return true;
 }
 
-void SMsgReader::readExtendedClipboard(rdr::S32 len)
+bool SMsgReader::readExtendedClipboard(int32_t len)
 {
-  rdr::U32 flags;
-  rdr::U32 action;
+  uint32_t flags;
+  uint32_t action;
+
+  if (!is->hasData(len))
+    return false;
 
   if (len < 4)
-    throw Exception("Invalid extended clipboard message");
+    throw protocol_error("Invalid extended clipboard message");
   if (len > maxCutText) {
     vlog.error("Extended clipboard message too long (%d bytes) - ignoring", len);
     is->skip(len);
-    return;
+    return true;
   }
 
   flags = is->readU32();
@@ -246,7 +368,7 @@ void SMsgReader::readExtendedClipboard(rdr::S32 len)
   if (action & clipboardCaps) {
     int i;
     size_t num;
-    rdr::U32 lengths[16];
+    uint32_t lengths[16];
 
     num = 0;
     for (i = 0;i < 16;i++) {
@@ -254,8 +376,8 @@ void SMsgReader::readExtendedClipboard(rdr::S32 len)
         num++;
     }
 
-    if (len < (rdr::S32)(4 + 4*num))
-      throw Exception("Invalid extended clipboard message");
+    if (len < (int32_t)(4 + 4*num))
+      throw protocol_error("Invalid extended clipboard message");
 
     num = 0;
     for (i = 0;i < 16;i++) {
@@ -270,7 +392,7 @@ void SMsgReader::readExtendedClipboard(rdr::S32 len)
     int i;
     size_t num;
     size_t lengths[16];
-    rdr::U8* buffers[16];
+    uint8_t* buffers[16];
 
     zis.setUnderlying(is, len - 4);
 
@@ -279,21 +401,45 @@ void SMsgReader::readExtendedClipboard(rdr::S32 len)
       if (!(flags & 1 << i))
         continue;
 
+      if (!zis.hasData(4))
+        throw protocol_error("Extended clipboard decode error");
+
       lengths[num] = zis.readU32();
+
       if (lengths[num] > (size_t)maxCutText) {
         vlog.error("Extended clipboard data too long (%d bytes) - ignoring",
                    (unsigned)lengths[num]);
-        zis.skip(lengths[num]);
+
+        // Slowly (safely) drain away the data
+        while (lengths[num] > 0) {
+          size_t chunk;
+
+          if (!zis.hasData(1))
+            throw protocol_error("Extended clipboard decode error");
+
+          chunk = zis.avail();
+          if (chunk > lengths[num])
+            chunk = lengths[num];
+
+          zis.skip(chunk);
+          lengths[num] -= chunk;
+        }
+
         flags &= ~(1 << i);
+
         continue;
       }
 
-      buffers[num] = new rdr::U8[lengths[num]];
+      if (!zis.hasData(lengths[num]))
+        throw protocol_error("Extended clipboard decode error");
+
+      buffers[num] = new uint8_t[lengths[num]];
       zis.readBytes(buffers[num], lengths[num]);
       num++;
     }
 
-    zis.removeUnderlying();
+    zis.flushUnderlying();
+    zis.setUnderlying(nullptr, 0);
 
     handler->handleClipboardProvide(flags, lengths, buffers);
 
@@ -309,37 +455,59 @@ void SMsgReader::readExtendedClipboard(rdr::S32 len)
       handler->handleClipboardRequest(flags);
       break;
     case clipboardPeek:
-      handler->handleClipboardPeek(flags);
+      handler->handleClipboardPeek();
       break;
     case clipboardNotify:
       handler->handleClipboardNotify(flags);
       break;
     default:
-      throw Exception("Invalid extended clipboard action");
+      throw protocol_error("Invalid extended clipboard action");
     }
   }
+
+  return true;
 }
 
-void SMsgReader::readQEMUMessage()
+bool SMsgReader::readQEMUMessage()
 {
-  int subType = is->readU8();
+  int subType;
+  bool ret;
+
+  if (!is->hasData(1))
+    return false;
+
+  is->setRestorePoint();
+
+  subType = is->readU8();
+
   switch (subType) {
   case qemuExtendedKeyEvent:
-    readQEMUKeyEvent();
+    ret = readQEMUKeyEvent();
     break;
   default:
-    throw Exception("unknown QEMU submessage type %d", subType);
+    throw protocol_error(format("Unknown QEMU submessage type %d", subType));
+  }
+
+  if (!ret) {
+    is->gotoRestorePoint();
+    return false;
+  } else {
+    is->clearRestorePoint();
+    return true;
   }
 }
 
-void SMsgReader::readQEMUKeyEvent()
+bool SMsgReader::readQEMUKeyEvent()
 {
+  if (!is->hasData(2 + 4 + 4))
+    return false;
   bool down = is->readU16();
-  rdr::U32 keysym = is->readU32();
-  rdr::U32 keycode = is->readU32();
+  uint32_t keysym = is->readU32();
+  uint32_t keycode = is->readU32();
   if (!keycode) {
     vlog.error("Key event without keycode - ignoring");
-    return;
+    return true;
   }
   handler->keyEvent(keysym, keycode, down);
+  return true;
 }
