@@ -47,8 +47,8 @@ using namespace winvnc;
 
 static LogWriter vlog("STrayIcon");
 
-BoolParameter STrayIconThread::disableOptions("DisableOptions", "Disable the Options entry in the VNC Server tray menu.", false);
-BoolParameter STrayIconThread::disableClose("DisableClose", "Disable the Close entry in the VNC Server tray menu.", false);
+BoolParameter STrayIconThread::disableOptions("DisableOptions", "Disable the Options entry in the VNC server tray menu.", false);
+BoolParameter STrayIconThread::disableClose("DisableClose", "Disable the Close entry in the VNC server tray menu.", false);
 
 
 //
@@ -73,20 +73,20 @@ namespace winvnc {
 class STrayIcon : public TrayIcon {
 public:
   STrayIcon(STrayIconThread& t) :
-    vncConfig(_T("vncconfig.exe"), isServiceProcess() ? _T("-noconsole -service") : _T("-noconsole")),
-    vncConnect(_T("winvnc4.exe"), _T("-noconsole -connect")), thread(t) {
+    vncConfig("vncconfig.exe", isServiceProcess() ? "-noconsole -service" : "-noconsole"),
+    vncConnect("winvnc4.exe", "-noconsole -connect"), thread(t) {
 
     // ***
-    SetWindowText(getHandle(), _T("winvnc::IPC_Interface"));
+    SetWindowText(getHandle(), "winvnc::IPC_Interface");
     // ***
 
-    SetTimer(getHandle(), 1, 3000, 0);
+    SetTimer(getHandle(), 1, 3000, nullptr);
     PostMessage(getHandle(), WM_TIMER, 1, 0);
     PostMessage(getHandle(), WM_SET_TOOLTIP, 0, 0);
     CPanel = new ControlPanel(getHandle());
   }
 
-  virtual LRESULT processMessage(UINT msg, WPARAM wParam, LPARAM lParam) {
+  LRESULT processMessage(UINT msg, WPARAM wParam, LPARAM lParam) override {
     switch(msg) {
 
     case WM_USER:
@@ -100,7 +100,7 @@ public:
           SendMessage(getHandle(), WM_COMMAND, ID_CONTR0L_PANEL, 0);
           break;
         case WM_RBUTTONUP:
-          HMENU menu = LoadMenu(GetModuleHandle(0), MAKEINTRESOURCE(thread.menu));
+          HMENU menu = LoadMenu(GetModuleHandle(nullptr), MAKEINTRESOURCE(thread.menu));
           HMENU trayMenu = GetSubMenu(menu, 0);
 
 
@@ -123,7 +123,7 @@ public:
           // Display the menu
           POINT pos;
           GetCursorPos(&pos);
-          TrackPopupMenu(trayMenu, 0, pos.x, pos.y, 0, getHandle(), 0);
+          TrackPopupMenu(trayMenu, 0, pos.x, pos.y, 0, getHandle(), nullptr);
 
           break;
 		} 
@@ -154,13 +154,13 @@ public:
         thread.server.disconnectClients("tray menu disconnect");
         break;
       case ID_CLOSE:
-        if (MsgBox(0, _T("Are you sure you want to close the server?"),
+        if (MsgBox(nullptr, "Are you sure you want to close the server?",
                    MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDYES) {
           if (isServiceProcess()) {
             try {
               rfb::win32::stopService(VNCServerService::Name);
-            } catch (rdr::Exception& e) {
-              MsgBox(0, TStr(e.str()), MB_ICONERROR | MB_OK);
+            } catch (std::exception& e) {
+              MsgBox(nullptr, e.what(), MB_ICONERROR | MB_OK);
             }
           } else {
             thread.server.stop();
@@ -180,15 +180,14 @@ public:
         switch (command->dwData) {
         case 1:
           {
-            CharArray viewer(command->cbData + 1);
-            memcpy(viewer.buf, command->lpData, command->cbData);
-            viewer.buf[command->cbData] = 0;
-            return thread.server.addNewClient(viewer.buf) ? 1 : 0;
+            std::string viewer((char*)command->lpData, command->cbData);
+            return thread.server.addNewClient(viewer.c_str()) ? 1 : 0;
           }
         case 2:
           return thread.server.disconnectClients("IPC disconnect") ? 1 : 0;
         case 3:
           thread.server.setClientsStatus(&CPanel->ListConnStatus);
+          /* fall through */
         case 4:
           thread.server.getClientsInfo(&LCInfo);
           CPanel->UpdateListView(&LCInfo);
@@ -219,8 +218,8 @@ public:
     case WM_SET_TOOLTIP:
       {
         os::AutoMutex a(thread.lock);
-        if (thread.toolTip.buf)
-          setToolTip(thread.toolTip.buf);
+        if (!thread.toolTip.empty())
+          setToolTip(thread.toolTip.c_str());
       }
       return 0;
 
@@ -240,7 +239,7 @@ protected:
 
 STrayIconThread::STrayIconThread(VNCServerWin32& sm, UINT inactiveIcon_, UINT activeIcon_, 
                                  UINT dis_inactiveIcon_, UINT dis_activeIcon_, UINT menu_)
-: thread_id(-1), windowHandle(0), server(sm),
+: thread_id(-1), windowHandle(nullptr), server(sm),
   inactiveIcon(inactiveIcon_), activeIcon(activeIcon_),
   dis_inactiveIcon(dis_inactiveIcon_), dis_activeIcon(dis_activeIcon_),
   menu(menu_), runTrayIcon(true) {
@@ -267,20 +266,19 @@ void STrayIconThread::worker() {
     windowHandle = icon.getHandle();
 
     MSG msg;
-    while (runTrayIcon && ::GetMessage(&msg, 0, 0, 0) > 0) {
+    while (runTrayIcon && ::GetMessage(&msg, nullptr, 0, 0) > 0) {
       TranslateMessage(&msg);
       DispatchMessage(&msg);
     }
 
-    windowHandle = 0;
+    windowHandle = nullptr;
   }
 }
 
-void STrayIconThread::setToolTip(const TCHAR* text) {
+void STrayIconThread::setToolTip(const char* text) {
   if (!windowHandle) return;
   os::AutoMutex a(lock);
-  delete [] toolTip.buf;
-  toolTip.buf = tstrDup(text);
+  toolTip = text;
   PostMessage(windowHandle, WM_SET_TOOLTIP, 0, 0);
 }
 
